/* faddr - fidonet address handling */
/* Written by Paul Edwards, 1993-12-11 */
/* Released to the Public Domain */

/* This set of routines is designed to allow fido-addresses that are 
   abbreviated to be expanded. */


#include <stdio.h>
#include <stdlib.h>

#include "faddr.h"
#include "prseaddr.h"
#include "unused.h"
        

/* defaults - set the default address of the user.
   default domain is fidonet, since that's the largest
   default zone is 1, since it's the largest
   default net is 1, since it's the most important I think
   default node is 1, since we're on a roll
   default point is 0, since that's the BBS proper */
   
int faddrDefaults(FADDR *faddr)
{
  strcpy(faddr->curr.domain, "fidonet");
  faddr->curr.zone = 1;
  faddr->curr.net = 1;
  faddr->curr.node = 1;
  faddr->curr.point = 0;
  return (0);
}


/* init - using whatever address the user has supplied (via the various
set... calls), make this the default, for whenever a reset is done. */

int faddrInit(FADDR *faddr)
{
  faddr->orig = faddr->curr;
  return (0);
}


/* term - terminate and free up resources */

int faddrTerm(FADDR *faddr)
{
  unused(faddr);
  return (0);
}


/* reset - return to state after init was done */

int faddrReset(FADDR *faddr)
{
  faddr->curr = faddr->orig;
  return (0);
}


/* getaddr - return the address that has been constructed */

int faddrGetaddr(FADDR *faddr, fidoAddress *fa)
{
  *fa = faddr->curr;
  return (0);
}

/* create - create a fadd based on a character string of fido address */

void faddrCreate(FADDR *faddr, char *buf)
{
    fidoAddress fa;
    int hasDomain;
    int hasZone;
    int hasNet;
    int hasNode;
    int hasPoint;
    
    prseaddr(buf,
             fa.domain,
             &hasDomain, 
             &fa.zone, 
             &hasZone,
             &fa.net, 
             &hasNet,
             &fa.node, 
             &hasNode,
             &fa.point,
             &hasPoint);
    if (hasDomain)
    {
        faddrSetdomain(faddr, fa.domain);
    }
    if (hasZone)
    {
        faddrSetzone(faddr, fa.zone);
    }
    if (hasNet)
    {
        faddrSetnet(faddr, fa.net);
    }
    if (hasNode)
    {
        faddrSetnode(faddr, fa.node);
    }
    if (hasPoint)
    {
        faddrSetpoint(faddr, fa.point);
    }
    return;
}        

    
/* print - print out faddr */

void faddrPrint(FADDR *faddr)
{
    printf("address is %d:%d/%d.%d@%s\n",
           faddr->curr.zone,
           faddr->curr.net,
           faddr->curr.node,
           faddr->curr.point,
           faddr->curr.domain);
    return;
}           
           

