/*********************************************************************/
/*                                                                   */
/*  This Program Written by Paul Edwards, 3:711/934@fidonet.         */
/*  Released to the Public Domain                                    */
/*                                                                   */
/*********************************************************************/
/*********************************************************************/
/*                                                                   */
/*  findkblk - find a kludge block.                                  */
/*                                                                   */
/*  Fidonet packets have at least 2 kludges - SEENBY and PATH,       */
/*  which occur in blocks.  This routine is designed to help you     */
/*  extract these kludge blocks.  top should point to the start of   */
/*  the text, which, if reached, means that there are too many       */
/*  kludges of this sort, return code 2.  If there were no           */
/*  occurences of the kludge, you get return code 1.  Otherwise,     */
/*  you get a return code of 0, st will point to the start of the    */
/*  kludges, en will point to the end of the kludges (ie after the   */
/*  last 0x0d), and num will contain the number of kludges found.    */
/*  If the return code is 2, st and end will also be set.            */
/*                                                                   */
/*  bottom should point to the bottom of the text, which should be   */
/*  a NUL.  srch contains the kludge you are looking for, which      */
/*  should occur straight after the 0x0d/0x0a.  lens contains the    */
/*  length of the search string.                                     */
/*                                                                   */
/*********************************************************************/

#include <string.h>

#include "findkblk.h"

int findkblk(unsigned char *top, 
             unsigned char *bottom, 
             unsigned char **st, 
             unsigned char **en, 
             int *num,
             unsigned char *srch,
             size_t lens)
{
    unsigned char *p;
    unsigned char *tmpst;
    int ret;
    
    *num = 0;
    p = bottom - 1;
    /* find a CR */
    while ((*p != 0x0d) && (p > top))
    {
        p--;
    }
    if (*p == 0x0d)
    {
        *en = p + 1;
        if (**en == 0x0a)
        {
            (*en)++;
        }
        p--;
        while ((*num == 0) && (p > top))
        {
            while ((*p != 0x0d) && (p > top))
            {
                p--;
            }
            *st = p + 1;
            if (**st == 0x0a)
            {
                (*st)++;
            }
            if (memcmp(*st, srch, lens) == 0)
            {
                (*num)++;
            }
            else
            {
                *en = *st;
            }
            p--;
        }
        if (*num > 0)
        {
            while (p > top)
            {
                while ((*p != 0x0d) && (p > top))
                {
                    p--;
                }
                if (p > top)
                {
                    tmpst = p + 1;
                    if (*tmpst == 0x0a)
                    {
                        tmpst++;
                    }
                    if (memcmp(tmpst, srch, lens) == 0)
                    {
                        (*num)++;
                        *st = tmpst;
                    }
                    else
                    {
                        break;
                    }
                }
                p--;
            }
        }
    } /* if found first (ie last) 0x0d */
    if (*num == 0)
    {
        ret = 1;
    }
    else if (p <= top)
    {
        ret = 2;
    }
    else
    {
        ret = 0;
    }
    return (ret);    
}
