/* mailproc - mailprocessing for Tobruk */
/* Originally written by Paul Edwards, 1993-12-30 */
/* Released to the Public Domain */

#ifndef MAILPROC_INCLUDED
#define MAILPROC_INCLUDED

#include <time.h>

#ifdef USE_MBASE
#include "mbase.h"
#endif

#ifdef USE_MSGAPI
#include <msgapi.h>
#define MSGAPI_OPEN "MPR001 failed to open msgapi, msgapierr is %d\n"
#define MSG_AREA_OPEN "MPR002 failed to open area %s, msgapierr is %d\n"
#define MSG_MSG_CREATE "MPR003 failed to write message, msgapierr is %d\n"
#define MSG_READ "MPR004 failed to read message, msgapierr is %d\n"
#endif

#include "faddr.h"
#include "seenby.h"
#include "estdio.h"
#include "pkt.h"
#include "area.h"
#include "route.h"

#define MAILPROC_SEENBY_MAX 500
#define MAILPROC_PATH_MAX 50
#define MAILPROC_MAX_DESTS 50

#define MAILPROC_MB_SQUISH 0
#define MAILPROC_MB_STAR 1

#define MAILPROC_MSG_NETMAIL \
    "MLP001 Warning - netmail message being argued over in packet %s\n"
#define MAILPROC_SETVBUF \
    "MLP002 Failed to allocate memory of %ld bytes\n"    
#define MAILPROC_TOO_MANY_DESTS \
    "MLP003 Maximum number of destinations (%d) exceeded\n"
#define MAILPROC_BAD_MSGSTART \
    "MLP004 Bad characters at message start : %x and %x\n" 
#define MAILPROC_NO_ORIGIN \
    "MLP005 Origin line missing in message %d\n"
#define MAILPROC_MSG_WRITE \
    "MLP006 Failed to write message number %d\n"    
#define MAILPROC_BADMSG \
    "MLP007 Error occurred in message number %d\n"
#define MAILPROC_RENAME \
    "MLP008 Failed to rename packet %s from pattern %s to pattern %s\n"    
#define MAILPROC_UNKNOWN_AREA \
    "MLP009 Unknown area %s\n"    
#define MAILPROC_FROM \
    "MLP010 From: %s\n"
#define MAILPROC_TO \
    "MLP011 To: %s\n"
#define MAILPROC_SUBJECT \
    "MLP012 Subject: %s\n"
#define MAILPROC_CONFERENCE \
    "MLP013 Conference is: %s\n"    
#define MAILPROC_ORIGIN \
    "MLP014 Origin line is: %s\n"
#define MAILPROC_MSGORIGIN \
    "MLP015 Message came from %d:%d/%d.%d\n"    
#define MAILPROC_NO_SEENBY \
    "MLP016 No seenbys in message\n"
#define MAILPROC_TOO_MANY_SEENBY \
    "MLP017 Too many seenbys in message\n"        
#define MAILPROC_TOO_MANY_PATH \
    "MLP017 Too many path lines in message\n"        
#define MAILPROC_MSGWRITE_FAIL \
    "MLP018 Failed to write message - msgapierr is" \
    " %d, total length is %ld, text length is %ld, control length is %ld\n"
#define MAILPROC_FUCKED_DATE \
    "MLP019 Incredibly fucked date - %s\n"    
#define MAILPROC_PKT_NOT_MINE \
    "MLP020 Packet %s is not for this node\n"    
#define MAILPROC_ECHOMAIL_ORIGIN \
    "MLP021 This echomail message has come to me routed\n"
#define MAILPROC_ECHOMAIL_DEST \
    "MLP022 An attempt has been made to route echomail through me\n"        
#define MAILPROC_MSGWRITE_FAIL2 \
    "MLP023 Failed to write message - msgapierr is %d\n"

typedef struct {
    int year;
    int month;
    int day;
    int hour;
    int min;
    int sec;
} dt;

typedef struct {
    unsigned char origNode[2];
    unsigned char destNode[2];
    unsigned char origNet[2];
    unsigned char destNet[2];
    unsigned char Attribute[2];
    unsigned char cost[2];
} raw_fido_msg_hdr;

typedef struct {
    int origNode;
    int destNode;
    int origNet;
    int destNet;
    unsigned int Attribute;
    int cost;
} fido_msg_hdr;

typedef struct {
    unsigned char origNode[2];
    unsigned char destNode[2];
    unsigned char year[2];
    unsigned char month[2];
    unsigned char day[2];
    unsigned char hour[2];
    unsigned char minute[2];
    unsigned char second[2];
    unsigned char baud[2];
    unsigned char PacketType[2];
    unsigned char origNet[2];
    unsigned char destNet[2];
    unsigned char prodCode;
    unsigned char serialNo;
    unsigned char password[8];
    unsigned char origZone[2];
    unsigned char destZone[2];
    unsigned char auxNet[2];
    unsigned char CWvalidationCopy[2];
    unsigned char prodCodeH;
    unsigned char Revision;
    unsigned char CapabilWord[2];
    unsigned char origZone2[2];
    unsigned char destZone2[2];
    unsigned char origPoint[2];
    unsigned char destPoint[2];
    unsigned char prodData[4];
    /* zero or more packed messages, followed by "\x00\x00\x00\x00"
       follow.*/
} raw_fido_packet_hdr;

typedef struct {
    unsigned int origNode;
    unsigned int destNode;
    dt pdate;
    unsigned int baud;
    unsigned int PacketType;
    unsigned int origNet;
    unsigned int destNet;
    unsigned int prodCode;
    unsigned int serialNo;
    unsigned char password[8];
    unsigned int origZone;
    unsigned int destZone;
    unsigned int capability;
    unsigned int origPoint;
    unsigned int destPoint;
    unsigned int Revision;
    unsigned char prodData[4];
} fido_packet_hdr;

int cvt_from_raw_packet_hdr(const raw_fido_packet_hdr *raw_p,
    fido_packet_hdr *fp_hdr);

int cvt_to_raw_packet_hdr(const fido_packet_hdr *fp_hdr,
    raw_fido_packet_hdr *raw_p);

int cvt_from_raw_message_hdr(const raw_fido_msg_hdr *raw_m,
    fido_msg_hdr *fm_hdr);

int cvt_to_raw_message_hdr(const fido_msg_hdr *fm_hdr,
    raw_fido_msg_hdr *raw_m);

typedef struct {
    struct {
        char echoName[50];
        int numAddr;
        fidoAddress *fa;
    } *echoList;  
    
    SEENBY seenby;
    unsigned char *startSeenby;
    unsigned char *endSeenby;
    int numSeenby;
    
    SEENBY path;
    unsigned char *startPath;
    unsigned char *endPath;
    int numPath;
    
    unsigned char tempControl[2000];
    size_t maxControl;
    
    char badName[FILENAME_MAX];
    char goodName[FILENAME_MAX];
    int seenbyMax;
    int pathMax;
    int debug;
    int okCount;
    char address[100];
    char name[100];
    char outbound[FILENAME_MAX];
    char packet[FILENAME_MAX];
    unsigned char toUserName[36];
    unsigned char fromUserName[36];
    unsigned char subject[72];
    unsigned char conference[75];
    unsigned char kludge[100];
    unsigned char origin[100];
    unsigned char seenbyLine[100];
    unsigned char pathLine[100];
    unsigned char newSeenby[69];
    unsigned char newPath[70];
    int msgnum;
    raw_fido_msg_hdr raw_m;
    fido_msg_hdr fm_hdr;
    int echoFlag;
    size_t mainbufSize;
    size_t mnbufsz;
    size_t mnbufhsz;
    unsigned char *mainbuf;
    EFILE packetFile;
    size_t inbufsize;
    void *inbuf; /* for use by setvbuf() for incoming packet */
    int numAreas;
    char lastArea[FILENAME_MAX];
    int lastAreaLen;
    int lastAreaNum;
    AREADEFN netarea;
    AREADEFN *areas;
    AREADEFN *currAreaDef;
    fidoAddress baseAddr; /* My address in this area */
    fidoAddress mainAddr; /* My address in the absence of an area override */
    fidoAddress worldRoute;
    struct {
        fidoAddress addr;
        char packetName[FILENAME_MAX];
        EFILE packetFile;
        int packetOpened;
    } dests[MAILPROC_MAX_DESTS];
    int destCount;
    int sequence;
    char newPacket[FILENAME_MAX];
    EFILE *currDest;
    fidoAddress *currAddress;
    int confAreaNum;
    int confDestNum;
#ifdef USE_MBASE
    MBASE mbase;
    ASET aset;
    size_t msglen;
    int allin;
    unsigned char *msgorig;
    GMD gmd;
#endif
#ifdef USE_MSGAPI
    struct _minf minf;
    MSGH *msgh;
    XMSG xmsg;
    dword highWater;
    int allin;
    dword msgoffs;
    dword msgbytes;
    size_t msglen;
    unsigned char *msgorig;
    int numhalves;
#endif        
    int totalmsgs;
    int totalmsgsOut;
    time_t startTime;
    
    fidoAddress authorAddr;
    fidoAddress recipAddr;
    fidoAddress msgOrigin;
    fidoAddress msgDest;
    fidoAddress pktOrigin;
    fidoAddress pktDest;
    
    dt mdate;
    PKT pkt;
    unsigned char *tptr;
    int areaCount;
    int routed;
    int remapped;
    fidoAddress routeAddr;
    int pointMap;
    ROUTE *route;
    int numRemaps;
    int remapCount;
    struct {
        char name[36];
        fidoAddress addr;
    } *remaps;
    
} MAILPROC;

void mailprocDefaults(MAILPROC *mailproc);
void mailprocInit(MAILPROC *mailproc);
void mailprocReset(MAILPROC *mailproc);
void mailprocTerm(MAILPROC *mailproc);
void mailprocPacket(MAILPROC *mailproc, char *packetName);
void mailprocNumAreas(MAILPROC *mailproc, int numAreas);
void mailprocAddArea(MAILPROC *mailproc, 
                     char *areaName, 
                     char *messageBase,
                     int numNodes,
                     int passThrough,
                     int mbType,
                     fidoAddress *primary);
void mailprocNumRemaps(MAILPROC *mailproc, int numRemaps);
void mailprocAddRemap(MAILPROC *mailproc, char *name, fidoAddress *addr);
void mailprocAddNodeOnly(MAILPROC *mailproc, FADDR *fidoAddr);
void mailprocAddNode(MAILPROC *mailproc, FADDR *faddr);
long mailprocTotalTime(MAILPROC *mailproc);
void mailprocNetmail(MAILPROC *mailproc, char *base, int mbType);
#define mailprocMainbufSize(mailproc, x) ((mailproc)->mainbufSize = x)
#define mailprocInbufsize(mailproc, x) ((mailproc)->inbufsize = x)
#define mailprocDebug(mailproc, x) ((mailproc)->debug = x)
#define mailprocOkCount(mailproc, x) ((mailproc)->okCount = x)
#define mailprocName(mailproc, x) (strcpy((mailproc)->name, x))
#define mailprocAddress(mailproc, x) (strcpy((mailproc)->address, x))
#define mailprocOutbound(mailproc, x) (strcpy((mailproc)->outbound, x))
#define mailprocBaseAddr(mailproc, x) ((mailproc)->baseAddr = *x)
#define mailprocWorldRoute(mailproc, x) ((mailproc)->worldRoute = *x)
#define mailprocTotalMsgs(mailproc) ((mailproc)->totalmsgs)
#define mailprocTotalMsgsOut(mailproc) ((mailproc)->totalmsgsOut)
#define mailprocBadName(mailproc, x) (strcpy((mailproc)->badName, x))
#define mailprocGoodName(mailproc, x) (strcpy((mailproc)->goodName, x))
#define mailprocPointMap(mailproc, x) ((mailproc)->pointMap = x)
#define mailprocSupplyRoute(mailproc, x) ((mailproc)->route = x)
void mailprocScanBase(MAILPROC *mailproc);

#endif


