/* Written by Paul Edwards */
/* Released to the Public Domain */
/* mbase - generic messagebase routines */

/* This is an implementation of the routines described in FSC-0502 */
/* When completed, it will have:

   1. The ability to use a pretty portable *.MSG implementation.
   2. The ability to use the Squish MSGAPI for *.MSG and Squish.
   3. The ability to use the MSGED routines for *.MSG and Quickbbs.
   
   Currently only number 1 exists, with zero error-checking.
   
   Current restrictions include only being able to access the last
   message area queried.
*/   

#include <stdio.h>
#include <stdlib.h>

#include "mbase.h"
#include "unused.h"
#include "area.h"
#include "trav.h"
#include "dostime.h"


/* Set up some defaults - should be called once by the application */

int mbaseDefaults(MBASE *mbase)
{
    unused(mbase);
    return (0);
}


/* Initialise the messagebase ready for use.  Should be called
   once by the application (not once for each area!) */

int mbaseInit(MBASE *mbase)
{
    unused(mbase);
    return (0);
}


/* Terminate the messagebase.  Should be called when the application
   no longer wishes to use the services of the messagebase */

int mbaseTerm(MBASE *mbase)
{
    unused(mbase);
    return (0);
}


/* retrieve the set of all areas (topics) known to the messagebase */

int mbaseGetAreaList(MBASE *mbase, ASET *aset)
{
    unused(mbase);
    unused(aset);
    return (0);
}


/* Find the specified area (by name) in the area set.  Returns
   the area handle in *gah.  Returns 0 if found, > 0 if not found,
   and < 0 on error */

int mbaseAsetQueryByName(MBASE *mbase, ASET *aset, GAH *gah, void *name)
{
    int x;

    unused(aset);    
    for (x = 0; x < mbase->numAreas; x++)
    {
        if (strcmp(mbase->echolist[x].areaName, name) == 0)
        {
            break;
        }
    }
    gah->areanum = x;
    if (x >= mbase->numAreas)
    {
        if (strcmp(name, "NETMAIL") != 0)
        {
            gah->areanum = -1;
        }
    }
    return (0);
}


/* discard a queried ASET */

int mbaseAsetDiscard(MBASE *mbase, ASET *aset)
{
    unused(aset);
    unused(mbase);
    return (0);
}


int mbaseAsetGetCount(MBASE *mbase, ASET *aset, long *count)
{
    unused(aset);
    *count = mbase->numAreas + 1; /* to allow for netmail */
    return (0);
}


/* Get the relative area number */

int mbaseAsetGetRel(MBASE *mbase, ASET *aset, GAH *gah, long relNum)
{
    unused(aset);
    unused(mbase);
    gah->areanum = relNum;
    return (0);
}


/* Given an area handle, retrieve the data associated with it */

int mbaseGetAreaData(MBASE *mbase, GAH *gah, GAD *gad)
{
    if (gah->areanum != -1)
    {
        if (gah->areanum >= mbase->numAreas)
        {
            gad->passThrough = 0;
        }
        else
        {
            gad->passThrough = mbase->echolist[(size_t)gah->areanum].passThrough;
        }
    }
    return (0);
}

static int getcount(char *file, void *udata)
{
    unused(file);
    (*(long *)udata)++;
    return (1);
}

static int addnum(char *file, void *udata)
{
    char *p;
    
    p = strrchr(file, '/');
    if (p == NULL)
    {
        p = strrchr(file, '\\');
    }
    if (p != NULL)
    {
        **(int **)udata = atoi(p + 1);
        (*(int **)udata)++;
    }
    return (1);
}

static int sortFunc(const void *a, const void *b)
{
    if (*(int *)a < *(int *)b)
    {
        return (-1);
    }
    else if (*(int *)a == *(int *)b)
    {
        return (0);
    }
    else 
    {
        return (1);
    }
}

/* Get a set of messages associated with a particular area */

int mbaseGetMessagesInArea(MBASE *mbase, GAH *gah, MSET *mset)
{
    char buf[FILENAME_MAX];
    int *p;
    char *dir;

    mset->count = 0;
    if (gah->areanum >= mbase->numAreas)
    {
        dir = mbase->netarea->messageBase;
    }
    else
    {
        dir = mbase->echolist[(size_t)gah->areanum].messageBase;
    }
    sprintf(buf, "%s/*.msg", dir);
    trav(buf, NULL, getcount, &mset->count);
    mset->list = malloc((size_t)(mset->count * sizeof(int)));
    p = mset->list;
    trav(buf, NULL, addnum, &p);
    qsort(mset->list, (size_t)mset->count, sizeof(int), sortFunc);
    mbase->lastarea = (size_t)gah->areanum;
    return (0);
}


/* Get count of number of messages in set */

int mbaseMsetGetCount(MBASE *mbase, MSET *mset, long *count)
{
    unused(mbase);
    *count = mset->count;
    return (0);
}


/* Get a particular message (handle) from a set of messages by
   it's relative position in the set (1 = first, 2 = second etc) */
   
int mbaseMsetGetRel(MBASE *mbase, MSET *mset, GMH *gmh, long relnum)
{
    unused(mbase);
    gmh->absNumber = mset->list[(size_t)relnum];
    return (0);
}


/* Discard a particular queried MSET */

int mbaseMsetDiscard(MBASE *mbase, MSET *mset)
{
    unused(mbase);
    free (mset->list);
    return (0);
}


/* Set the lastread pointer for a particular area, and a particular
   user */

int mbaseSetLastRead(MBASE *mbase, GAH *gah, GUH *guh, GMH *gmh)
{
    unused(mbase);
    unused(gah);
    unused(guh);
    unused(gmh);
    return (0);
}


/* Open a message from the messagebase, given it's handle */

int mbaseOpenMessage(MBASE *mbase,
                     GMH *gmh, 
                     GMD *gmd, 
                     int options)
{
    char buf[FILENAME_MAX];
    char *dir;

    unused(options);
    if (mbase->lastarea >= mbase->numAreas)
    {
        dir = mbase->netarea->messageBase;
    }
    else
    {
        dir = mbase->echolist[mbase->lastarea].messageBase;
    }
    sprintf(buf, 
            "%s/%d.msg", 
            dir,
            gmh->absNumber);
    gmd->fu = fopen(buf, "r+b");
    return (0);
}

/* Read a message from the messagebase */

int mbaseAccessMessage(MBASE *mbase,
                  GMD *gmd, 
                  int options, 
                  void *text,
                  size_t start_text,
                  size_t max_text,
                  size_t *actual_text)
{
    static unsigned char buf[190];
    size_t bytes;
    
    unused(mbase);
    unused(start_text);
    rewind(gmd->fu);
    if (options == MBASE_UPDHDR)
    {
        buf[186] = gmd->attr & 0xff;
        buf[187] = (gmd->attr >> 8) & 0xff;
        fwrite(buf, 1, sizeof buf, gmd->fu);
    }
    else
    {
        fread(buf, sizeof buf, 1, gmd->fu);
        
        gmd->to_2d.node = buf[166] | (buf[167] << 8);
        gmd->from_2d.node = buf[168] | (buf[169] << 8);
        gmd->from_2d.net = buf[172] | (buf[173] << 8);
        gmd->to_2d.net = buf[174] | (buf[175] << 8);
        gmd->to_2d.zone = 0;
        gmd->from_2d.zone = 0;
        gmd->to_2d.point = 0;
        gmd->from_2d.point = 0;
        
        gmd->from_4d = gmd->from_2d;
        gmd->to_4d = gmd->to_2d;
        gmd->orig_4d = gmd->from_4d;
        
        memcpy(gmd->from, buf, 36);
        gmd->from[36] = '\0';
        memcpy(gmd->to, buf + 36, 36);
        gmd->to[36] = '\0';
        memcpy(gmd->subj, buf + 72, 72);
        gmd->subj[36] = '\0';
        memcpy(gmd->create_time, buf + 144, 20);
        gmd->attr = *(buf + 186) | (*(buf + 187) << 8);
        if (text != NULL)
        {
            bytes = fread(text, 1, max_text, gmd->fu);
            if (actual_text != NULL)
            {
                *actual_text = bytes;
            }
            if (strncmp(text, "\x01INTL", 5) == 0)
            {
                gmd->to_4d.zone = gmd->to_2d.zone = atoi((char *)text + 5);
            }
        }
    }
    return (0);
}


int mbaseCloseMessage(MBASE *mbase,
                      GMD *gmd)
{
    unused(mbase);
    fclose(gmd->fu);
    return (0);
}              
        

static long highest;

static int getHighest2(char *file, void *ucb)
{
    long x;
    char *p;

    unused(ucb);    
    p = strrchr(file, '/');
    if (p == NULL)
    {
        p = strrchr(file, '\\');
    }
    if (p != NULL)
    {
        x = atol(p + 1);
        if (x > highest)
        {
            highest = x;
        }
    }
    return (1);
}

static void getHighest(char *dir, long *msgnum)
{
    highest = 0;
    trav(dir, NULL, getHighest2, NULL);
    *msgnum = highest;
    return;
}


/* Add a message to the messagebase, along with it's text */

int mbaseInsertMsg(MBASE *mbase, 
                   GMD *gmd, 
                   long totLen, 
                   unsigned char *text,
                   size_t textLen,
                   int fin)
{
    long msgnum;
    FILE *fq;
    char fnm[FILENAME_MAX];
    char *dir;
    int areanum;
    unsigned char dostime[4];

    unused(fin);
    unused(totLen);
    unused(textLen);
    areanum = (int)gmd->gah->areanum;
    if (areanum >= mbase->numAreas)
    {
        dir = mbase->netarea->messageBase;
    }
    else
    {
        dir = mbase->echolist[(size_t)gmd->gah->areanum].messageBase;
    }
    sprintf(fnm, "%s/*.msg", dir);
    getHighest(fnm, &msgnum);
    msgnum++;
    sprintf(fnm, 
            "%s/%ld.msg", 
            dir,
            msgnum);
    fq = fopen(fnm, "wb");
    if (fq == NULL) return (-1);
    fwrite(gmd->from, 36, 1, fq);
    fwrite(gmd->to, 36, 1, fq);
    fwrite(gmd->subj, 72, 1, fq);
    fwrite(gmd->create_time, 20, 1, fq);
    fwrite("\x00\x00", 2, 1, fq); /* times read */
    fputc(gmd->to_2d.node & 0xff, fq);
    fputc((gmd->to_2d.node >> 8) & 0xff, fq);
    fputc(gmd->from_2d.node & 0xff, fq);
    fputc((gmd->from_2d.node >> 8) & 0xff, fq);
    fwrite("\x00\x00", 2, 1, fq); /* cost */
    fputc(gmd->from_2d.net & 0xff, fq);
    fputc((gmd->from_2d.net >> 8) & 0xff, fq);
    fputc(gmd->to_2d.net & 0xff, fq);
    fputc((gmd->to_2d.net >> 8) & 0xff, fq);

#if 0
    fputc(gmd->to_2d.zone & 0xff, fq);
    fputc((gmd->to_2d.zone >> 8) & 0xff, fq);
    fputc(gmd->from_2d.zone & 0xff, fq);
    fputc((gmd->from_2d.zone >> 8) & 0xff, fq);
    fputc(gmd->to_2d.point & 0xff, fq);
    fputc((gmd->to_2d.point >> 8) & 0xff, fq);
    fputc(gmd->from_2d.point & 0xff, fq);
    fputc((gmd->from_2d.point >> 8) & 0xff, fq);
#else
    fwrite("\x00\x00\x00\x00", 4, 1, fq); /* written */
    /* arrived */
    timet_to_dos(time(NULL), dostime);
    fwrite(dostime, 4, 1, fq);
#endif    

    fwrite("\x00\x00", 2, 1, fq); /* reply to */
    fputc((int)(gmd->attr & 0xff), fq);
    fputc((int)((gmd->attr >> 8) & 0xff), fq);
    fwrite("\x00\x00", 2, 1, fq); /* next reply */
    fputs((char *)text, fq);
    fclose(fq);
    return (0);
}


/* This function is not part of the documented API */
/* It is required in this particular implementation in order */
/* to provide the list of areas, which are stored in memory */
/* It should be called after Defaults() and before Init() */

int mbaseProvideAreas(MBASE *mbase, 
                      AREADEFN *areas, 
                      long numAreas, 
                      AREADEFN *netarea)
{
    mbase->numAreas = numAreas;
    mbase->echolist = areas;
    mbase->netarea = netarea;
    return (0);
}
