/* written by Paul Edwards */
/* released to the Public Domain */
/* see pkt.c for documentation */

#ifndef PKT_INCLUDED
#define PKT_INCLUDED

#include "repofile.h"
#include "slidwin.h"

#define PKT_INVALID "PKT001 Packet is invalid, reason %d\n"

typedef struct {
    unsigned char *buf;
    size_t szbuf;
    int localmem;
    size_t szhbuf;
    size_t remain;
    REPOFILE repofile;
    SLIDWIN slidwin;
    long currLoc;
    long origLoc;
    size_t len1;
    int endOfPkt;
    long numHalves;
    
    unsigned char *startBuf; /* start of physical buffer */
    unsigned char *endBuf;   /* end of physical buffer */
    unsigned char *startPtr; /* start of message (points to message header) */
    unsigned char *endPtr;   /* end of message (points to NUL) */
    unsigned char *fendBuf;  /* end of logical buffer (points to dummy NUL) */
    unsigned char *startValT; /* start of first available exposed text */
    
    unsigned char *date;
    unsigned char *to;
    unsigned char *from;
    unsigned char *subject;
    unsigned char *text;
} PKT;

void pktDefaults(PKT *pkt);
void pktSetSize(PKT *pkt, size_t szbuf);
void pktSetBuf(PKT *pkt, void *buf);
void pktInit(PKT *pkt);
void pktTerm(PKT *pkt);
void pktSetFileBuf(PKT *pkt, void *vbuf, size_t vbufSize);
void pktSetFile(PKT *pkt, char *filename);
void pktFinFile(PKT *pkt);
int pktFirstMsg(PKT *pkt);
int pktNextMsg(PKT *pkt);
void pktHalf(PKT *pkt);
void pktFull(PKT *pkt);
void pktEnd(PKT *pkt);
void pktBegin(PKT *pkt);

#endif
