/* Tobruk - a portable, public domain, FTN mailprocessor */

/* Original version of this module written by Paul Edwards 1993-01-14 */
/* Released to the Public Domain */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "tobruk.h"
#include "mailproc.h"
#include "getopts.h"
#include "error.h"
#include "faddr.h"
#include "prseaddr.h"
#include "strinl.h"
#include "stritr.h"
#include "estdio.h"
#include "estdlib.h"
#include "trav.h"
#include "unused.h"
#include "route.h"

static void main2(int argc, char **argv);
static void tobrukBaseAddr(TOBRUK *tobruk);
static void tobrukConfigRead(TOBRUK *tobruk);
static void tobrukConfigRead2(TOBRUK *tobruk);
static void tobrukDests(TOBRUK *tobruk);
static void tobrukCountAddresses(TOBRUK *tobruk);
static void tobrukAddAddresses(TOBRUK *tobruk);
static void tobrukParseEcho(TOBRUK *tobruk);
static int tobrukPacket(char *fnm, void *ucb);

int main(int argc, char **argv)
{
    int ret = EXIT_FAILURE;
    char dtstr[20];
    struct tm *tms;
    time_t tt;

    time(&tt);
    tms = localtime(&tt);
    sprintf(dtstr, "%.4d-%.2d-%.2d %.2d:%.2d:%.2d",
            tms->tm_year+1900, tms->tm_mon+1, tms->tm_mday,
            tms->tm_hour, tms->tm_min, tms->tm_sec);
    errorDefaults();
    errorInit();
    if (ALLOK)
    {
        errorSet(TOBRUK_MSG_STARTUP, dtstr);
        errorFlush();
        main2(argc, argv);
        if (ALLOK)
        {
            ret = EXIT_SUCCESS;
            errorSet(TOBRUK_MSG_SUCCESS);
        }
    }
    errorTerm();
    return (ret);
}

static void main2(int argc, char **argv)
{
    TOBRUK *tobruk;

    tobruk = emalloc(sizeof *tobruk);
    if (ALLOK)
    {
        tobrukDefaults(tobruk);
        tobrukInit(tobruk);
        if (ALLOK)
        {
            tobrukRun(tobruk, argc, argv);
            tobrukTerm(tobruk);
        }
    }
    return;
}

void tobrukDefaults(TOBRUK *tobruk)
{
    unused(tobruk);
    return;
}

void tobrukInit(TOBRUK *tobruk)
{
    tobrukReset(tobruk);
    return;
}

void tobrukReset(TOBRUK *tobruk)
{
    tobruk->gotaddr = 0;
    *tobruk->outbound = '\0';
    tobruk->okCount = 0;
    *tobruk->name = '\0';
    tobruk->numAreas = 0;
    tobruk->numRemaps = 0;
    tobruk->pointMap = 0;
    *tobruk->dests = '\0';
    *tobruk->route_name = '\0';
    return;
}    

void tobrukTerm(TOBRUK *tobruk)
{
    unused(tobruk);
    return;
}

void tobrukRun(TOBRUK *tobruk, int argc, char **argv)
{
    static TOBRUK_OPTS opts = 
        { 0, "tobruk.pkt", 0, 20000, "tobruk.cfg", "" };
    opt_t optTable[] = 
    {
        { "d", OPTBOOL, &opts.debug },
        { "p", OPTSTR, opts.packet },
        { "i", OPTINT, &opts.inbufsize },
        { "b", OPTINT, &opts.mainbufSize },
        { "c", OPTSTR, opts.config },
        { "r", OPTSTR, opts.badName },
        { NULL, 0, NULL }
    };
    fidoAddress x;
    long nummsgs, numsecs;
    
    if (argc < 2)
    {
        errorSet(TOBRUK_MSG_USAGE);
    }
    if (ALLOK)
    {
        getopts(argc, argv, optTable);
        tobruk->tobrukOpts = opts;
        if (ALLOK)
        {        
            tobrukConfigRead(tobruk);
            if (ALLOK)
            {
                mailprocDefaults(&tobruk->mailproc);
                mailprocMainbufSize(&tobruk->mailproc,
                                 tobruk->tobrukOpts.mainbufSize);
                mailprocInbufsize(&tobruk->mailproc,
                                  tobruk->tobrukOpts.inbufsize);
                faddrGetaddr(&tobruk->faddr, &x);
                mailprocBaseAddr(&tobruk->mailproc, &x);
                mailprocInit(&tobruk->mailproc);
                mailprocNumAreas(&tobruk->mailproc, tobruk->numAreas);
                mailprocNumRemaps(&tobruk->mailproc, tobruk->numRemaps);
                tobrukDests(tobruk);
                tobrukConfigRead2(tobruk);
                if (*tobruk->route_name != '\0')
                {
                    routeDefaults(&tobruk->route);
                    routeInit(&tobruk->route, tobruk->route_name);
                    mailprocSupplyRoute(&tobruk->mailproc, &tobruk->route);
                }
                mailprocDebug(&tobruk->mailproc,
                              tobruk->tobrukOpts.debug);         
                mailprocOkCount(&tobruk->mailproc,
                                tobruk->okCount);         
                mailprocName(&tobruk->mailproc,
                             tobruk->name);
                mailprocAddress(&tobruk->mailproc,
                                tobruk->addr);
                faddrReset(&tobruk->faddr);
                faddrCreate(&tobruk->faddr, tobruk->worldRoute);
                faddrGetaddr(&tobruk->faddr, &x);
                mailprocWorldRoute(&tobruk->mailproc, &x);
                mailprocPointMap(&tobruk->mailproc, tobruk->pointMap);
                mailprocOutbound(&tobruk->mailproc,
                                 tobruk->outbound);
                mailprocBadName(&tobruk->mailproc,
                                tobruk->tobrukOpts.badName);
                mailprocGoodName(&tobruk->mailproc,
                                 tobruk->tobrukOpts.packet);
#ifdef HAVE_GROW_HANDLES
                _grow_handles(50);
                _grow_handles(100);
                _grow_handles(500);
                _grow_handles(1000);
#endif                                 
                trav(tobruk->tobrukOpts.packet, 
                     NULL,
                     tobrukPacket,
                     &tobruk->mailproc);
                nummsgs = mailprocTotalMsgs(&tobruk->mailproc);
                numsecs = mailprocTotalTime(&tobruk->mailproc);     
                printf("processed %ld messages in %ld secs\n",
                    nummsgs, numsecs);
                if (numsecs > 0)
                {
                    printf("rate is %ld messages per sec\n",
                        nummsgs/numsecs);
                }
                nummsgs = mailprocTotalMsgsOut(&tobruk->mailproc);
                printf("%ld messages sent\n", nummsgs);
                if (numsecs > 0)
                {
                    printf("rate is %ld messages per sec\n",
                        nummsgs/numsecs);
                }
                mailprocScanBase(&tobruk->mailproc);  
                mailprocTerm(&tobruk->mailproc);
                if (*tobruk->route_name != '\0')
                {
                    routeTerm(&tobruk->route);
                }
            }
        }
        faddrTerm(&tobruk->faddr);
    }
    return;
}

static int tobrukPacket(char *fnm, void *ucb)
{
    errorSet(TOBRUK_PROCESS_PACKET, fnm);
    errorFlush();
    mailprocPacket(ucb, fnm);
    return (ALLOK);
}

static void tobrukBaseAddr(TOBRUK *tobruk)
{
    faddrDefaults(&tobruk->faddr);
    faddrCreate(&tobruk->faddr, tobruk->addr);
    faddrInit(&tobruk->faddr);
    return;
}        

static void tobrukConfigRead(TOBRUK *tobruk)
{
    efileOpen(&tobruk->configFile, tobruk->tobrukOpts.config, "r");
    if (ALLOK)
    {
        while (ALLOK
               && (efileGets(&tobruk->configFile,
                            tobruk->buf, 
                            sizeof tobruk->buf) != NULL))
        {
            strinl(tobruk->buf);
            stritr(tobruk->buf);
            if (strncmp(tobruk->buf, "EchoArea", 8) == 0)
            {
                tobruk->numAreas++;
            }
            if (strncmp(tobruk->buf, "Remap", 5) == 0)
            {
                tobruk->numRemaps++;
            }
            else if (strncmp(tobruk->buf, "Address", 7) == 0)
            {
                if (!tobruk->gotaddr)
                {
                    strinl(tobruk->buf);
                    stritr(tobruk->buf);
                    strcpy(tobruk->addr, 
                           tobruk->buf+7+strspn(tobruk->buf+7, " "));
                    tobrukBaseAddr(tobruk);
                    tobruk->gotaddr = 1;
                }
            }
#ifdef BINKLEYSTYLE            
            else if (strncmp(tobruk->buf, "Outbound", 8) == 0)
            {
                if (*tobruk->outbound == '\0')
                {
                    strinl(tobruk->buf);
                    stritr(tobruk->buf);
                    strcpy(tobruk->outbound,
                           tobruk->buf+8+strspn(tobruk->buf+8, " "));
                }
            }
#endif            
            else if (strncmp(tobruk->buf, "App Tobruk Outbound", 19) == 0)
            {
                strinl(tobruk->buf);
                stritr(tobruk->buf);
                strcpy(tobruk->outbound,
                       tobruk->buf+19+strspn(tobruk->buf+19, " "));
            } 
            else if (strncmp(tobruk->buf, "App Tobruk WorldRoute", 21) == 0)
            {
                strinl(tobruk->buf);
                stritr(tobruk->buf);
                strcpy(tobruk->worldRoute, 
                       tobruk->buf+21+strspn(tobruk->buf+21, " "));
            }
            else if (strncmp(tobruk->buf, "App Tobruk PointMap", 19) == 0)
            {
                strinl(tobruk->buf);
                stritr(tobruk->buf);
                tobruk->pointMap = 
                    atoi(tobruk->buf+19+strspn(tobruk->buf+19, " "));
            }
            else if (strncmp(tobruk->buf, "App Tobruk Name", 15) == 0)
            {
                strinl(tobruk->buf);
                stritr(tobruk->buf);
                strcpy(tobruk->name,
                       tobruk->buf+15+strspn(tobruk->buf+15, " "));
            }
            else if (strncmp(tobruk->buf, "App Tobruk Via", 14) == 0)
            {
                strinl(tobruk->buf);
                stritr(tobruk->buf);
                tobruk->okCount = 
                    atoi(tobruk->buf+14+strspn(tobruk->buf+14, " "));
            }
            else if (strncmp(tobruk->buf, "App Tobruk Dests", 16) == 0)
            {
                strinl(tobruk->buf);
                stritr(tobruk->buf);
                strcpy(tobruk->dests,
                       tobruk->buf+16+strspn(tobruk->buf+16, " "));
            }
            else if (strncmp(tobruk->buf, "App Tobruk Route", 16) == 0)
            {
                strinl(tobruk->buf);
                stritr(tobruk->buf);
                strcpy(tobruk->route_name,
                       tobruk->buf+16+strspn(tobruk->buf+16, " "));
            }
        }
    }
    return;
}

static void tobrukConfigRead2(TOBRUK *tobruk)
{
    FADDR faddr;
    fidoAddress fa;
    
    faddrDefaults(&faddr);
    faddrCreate(&faddr, tobruk->addr);
    faddrInit(&faddr);
    efileRewind(&tobruk->configFile);
    while (ALLOK
           && (efileGets(&tobruk->configFile,
                         tobruk->buf, 
                         sizeof tobruk->buf) != NULL))
    {
        if (strncmp(tobruk->buf, "EchoArea", 8) == 0)
        {
            tobruk->mbType = MAILPROC_MB_STAR;
            tobruk->passThrough = 0;
            tobrukParseEcho(tobruk);
            tobrukCountAddresses(tobruk);
            mailprocAddArea(&tobruk->mailproc,
                            tobruk->areaName,
                            tobruk->messageBase,
                            tobruk->numAddr,
                            tobruk->passThrough,
                            tobruk->mbType,
                            &tobruk->primary);
            tobrukAddAddresses(tobruk);                
        }
        else if (strncmp(tobruk->buf, "NetArea", 7) == 0)
        {
            tobruk->mbType = MAILPROC_MB_STAR;
            tobrukParseEcho(tobruk);
            mailprocNetmail(&tobruk->mailproc, 
                            tobruk->messageBase,
                            tobruk->mbType);
        }
        else if (strncmp(tobruk->buf, "Remap", 5) == 0)
        {
            char *p;
            char *q;
            
            strinl(tobruk->buf);
            stritr(tobruk->buf);
            faddrReset(&tobruk->faddr);
            p = strtok(tobruk->buf + 5, " ");
            if (p != NULL)
            {
                q = strtok(NULL, "");
                if (q != NULL)
                {
                    q = q + strspn(q, " ");
                    faddrReset(&faddr);
                    faddrCreate(&faddr, p);
                    faddrGetaddr(&faddr, &fa);
                    mailprocAddRemap(&tobruk->mailproc, q, &fa);
                }
            }
        }
    }
    efileClose(&tobruk->configFile);
    faddrTerm(&faddr);
    return;
}

static void tobrukDests(TOBRUK *tobruk)
{
    char buf[200];
    
    if (strcmp(tobruk->dests, "") != 0)
    {
        efileOpen(&tobruk->destFile, tobruk->dests, "r");
        if (ALLOK)
        {
            while (efileGets(&tobruk->destFile, buf, sizeof buf) != NULL)
            {
                strinl(buf);
                stritr(buf);
                if (!isspace(*buf) && (*buf != ';'))
                {
                    faddrReset(&tobruk->faddr);
                    faddrCreate(&tobruk->faddr, buf);
                    mailprocAddNodeOnly(&tobruk->mailproc, &tobruk->faddr);
                }
            }
            efileClose(&tobruk->destFile);
        }
    }
    return;
}

static void tobrukParseEcho(TOBRUK *tobruk)
{
    char *pos;
    char *pos2;
    FADDR faddr;

    faddrOnePassCreate(&faddr, tobruk->addr, &tobruk->primary);
    pos = tobruk->buf;
    while (!isspace(*pos) && (*pos != '\0'))
    {
        pos++;
    }
    while (isspace(*pos))
    {
        pos++;
    }
    pos2 = pos;
    while (!isspace(*pos) && (*pos != '\0'))
    {
        pos++;
    }
    memcpy(tobruk->areaName, pos2, (size_t)(pos - pos2));
    *(tobruk->areaName + (size_t)(pos - pos2)) = '\0';
    while (isspace(*pos) && (*pos != '\0'))
    {
        pos++;
    }
    pos2 = pos;
    while (!isspace(*pos) && (*pos != '\0'))
    {
        pos++;
    }
    memcpy(tobruk->messageBase, pos2, (size_t)(pos - pos2));
    *(tobruk->messageBase + (size_t)(pos - pos2)) = '\0';
    while (isspace(*pos) && (*pos != '\0'))
    {
        pos++;
    }
    while (*pos == '-')
    {
        pos++;
        if (*pos == '$')
        {
            tobruk->mbType = MAILPROC_MB_SQUISH;
        }
        else if (*pos == '0')
        {
            tobruk->passThrough = 1;
        }
        else if (*pos == 'p')
        {
            faddrOnePassCreate(&faddr, pos + 1, &tobruk->primary);
        }
        while (!isspace(*pos) && (*pos != '\0'))
        {
            pos++;
        }
        while (isspace(*pos))
        {
            pos++;
        }
    }
    tobruk->nodeStart = pos;
    return;
}

static void tobrukCountAddresses(TOBRUK *tobruk)
{
    char *ptr;
  
    ptr = tobruk->nodeStart;
    if (!isspace(*ptr) && (*ptr != '\0'))
    {
        tobruk->numAddr = 1;
    }
    else
    {
        tobruk->numAddr = 0;
    }
    ptr = strchr(ptr, ' ');
    while (ptr != NULL)
    {
        tobruk->numAddr++;
        ptr += strspn(ptr, " ");
        ptr = strchr(ptr, ' ');
    }
    return;
}

static void tobrukAddAddresses(TOBRUK *tobruk)
{
    char *ptr;
  
    faddrReset(&tobruk->faddr);
    ptr = tobruk->nodeStart;
    while (isprint(*ptr))
    {
        faddrCreate(&tobruk->faddr, ptr);
        mailprocAddNode(&tobruk->mailproc, &tobruk->faddr);
        ptr = strchr(ptr, ' ');
        if (ptr == NULL)
        {
            break;
        }
        ptr += strspn(ptr, " ");
    }
    return;
}
