/* Originally written by Paul Edwards on 1993-01-14 */
/* Released to the Public Domain */

#ifndef TOBRUK

#include <stdio.h>

#include "mailproc.h"
#include "route.h"
#include "faddr.h"

#define TOBRUK_MAX_CONFIG 3000  /* maximum line length of config file */

#define TOBRUK_MSG_SUCCESS "TBK000 Tobruk completed successfully\n"
#define TOBRUK_MSG_USAGE                                     \
    "Tobruk - a public domain Fidonet mailprocessor\n\n"     \
    "options:\n"                                             \
    "-d debug?\n"                                            \
    "-p packet name\n"                                       \
    "-r rename bad packets according to this pattern\n"      \
    "-i how many bytes for input file buffer\n"              \
    "-b how many bytes to allocate for the main buffer\n"    \
    "-c name of configuration file\n"
#define TOBRUK_MSG_STARTUP \
    "TBK002 Tobruk Version 1.00 started at %s\n"
#define TOBRUK_PROCESS_PACKET \
    "TBK003 Processing packet %s\n"    

typedef struct {
    int  debug;
    char packet[FILENAME_MAX];
    int  inbufsize;
    int  mainbufSize;
    char config[FILENAME_MAX];
    char badName[FILENAME_MAX];
} TOBRUK_OPTS;    

typedef struct {
    char addr[40];
    char worldRoute[40];
    char dests[FILENAME_MAX];
    EFILE destFile;
    char name[80];
    int  gotaddr;
    int  okCount;
    char outbound[FILENAME_MAX];
    char tbkOutbound[FILENAME_MAX];
    char buf[TOBRUK_MAX_CONFIG];
    int numAreas;
    int numRemaps;
    char areaName[200];
    fidoAddress primary;
    char messageBase[FILENAME_MAX];
    TOBRUK_OPTS tobrukOpts;
    EFILE configFile;
    int numAddr;
    MAILPROC mailproc;
    FADDR faddr;
    char route_name[FILENAME_MAX];
    ROUTE route;
    char *nodeStart;
    int mbType;
    int passThrough;
    int pointMap;
} TOBRUK;

void tobrukDefaults(TOBRUK *tobruk);
void tobrukInit(TOBRUK *tobruk);
void tobrukReset(TOBRUK *tobruk);
void tobrukTerm(TOBRUK *tobruk);
void tobrukRun(TOBRUK *tobruk, int argc, char **argv);

#endif

